<?php

/**
 * @copyright Copyright (c) Metaways Infosystems GmbH, 2011
 * @license LGPLv3, http://www.arcavias.com/en/license
 */


/**
 * Adds foreign key constraint for langid to media table.
 */
class MW_Setup_Task_MediaAddForeignKey extends MW_Setup_Task_Abstract
{
	private $_mysql = array(

		'mshop_media' => array(
			'ALTER TABLE `mshop_media` ADD CONSTRAINT `fk_msmed_langid` FOREIGN KEY (`langid`) REFERENCES `mshop_global_language` (`id`) ON DELETE CASCADE ON UPDATE CASCADE',
		),
	);

	/**
	 * Returns the list of task names which this task depends on.
	 *
	 * @return array List of task names
	 */
	public function getPreDependencies()
	{
		return array();
	}


	/**
	 * Returns the list of task names which depends on this task.
	 *
	 * @return string[] List of task names
	 */
	public function getPostDependencies()
	{
		return array('TablesCreateMShop');
	}


	/**
	 * Executes the task for MySQL databases.
	 */
	protected function _mysql()
	{
		// Superseded by MediaDropLocaleConstraints
		// $this->_process( $this->_mysql );
	}


	/**
	 * Migrates service text data to list table.
	 *
	 * @param array $stmts Associative array of tables names and lists of SQL statements to execute.
	 */
	protected function _process( array $stmts )
	{
		$this->_msg( 'Adding foreign keys to mshop media table', 0 ); $this->_status( '' );

		foreach( $stmts as $table => $stmtList )
		{
			$this->_msg( sprintf( 'Checking table "%1$s": ', $table ), 1 );

			if ( $this->_schema->tableExists( $table ) === true
				&& $this->_schema->constraintExists( $table, 'fk_msmed_langid' ) === false
			)
			{
				$this->_executeList( $stmtList );
				$this->_status( 'migrated' );
			}
			else
			{
				$this->_status( 'OK' );
			}
		}
	}
}
