<?php

/**
 * @copyright Copyright (c) Metaways Infosystems GmbH, 2012
 * @license LGPLv3, http://www.arcavias.com/en/license
 */


/**
 * Test class for MShop_Coupon_Provider_Present.
 * Generated by PHPUnit on 2008-08-06 at 13:07:47.
 */
class MShop_Coupon_Provider_PresentTest extends MW_Unittest_Testcase
{
	private $_object;
	private $_orderBase;


	/**
	 * Sets up the fixture, for example, opens a network connection.
	 * This method is called before a test is executed.
	 *
	 * @access protected
	 */
	protected function setUp()
	{
		$context = TestHelper::getContext();

		$priceManager = MShop_Price_Manager_Factory::createManager( $context );
		$couponItem = MShop_Coupon_Manager_Factory::createManager( $context )->createItem();
		$couponItem->setConfig( array( 'present.productcode' => 'U:PD', 'present.quantity' => '1' ) );

		// Don't create order base item by createItem() as this would already register the plugins
		$this->_orderBase = new MShop_Order_Item_Base_Default( $priceManager->createItem(), $context->getLocale() );
		$this->_object = new MShop_Coupon_Provider_Present( $context, $couponItem, 'zyxw' );
	}


	/**
	 * Tears down the fixture, for example, closes a network connection.
	 * This method is called after a test is executed.
	 *
	 * @access protected
	 */
	protected function tearDown()
	{
		unset( $this->_object );
		unset( $this->_orderBase );
	}


	public function testAddCoupon()
	{
		$this->_object->addCoupon( $this->_orderBase );

		$coupons = $this->_orderBase->getCoupons();
		$products = $this->_orderBase->getProducts();

		if( !isset( $coupons['zyxw'][0] ) ) {
			throw new Exception( 'Missing coupon product' );
		}
		$product = $coupons['zyxw'][0];

		$this->assertEquals( 1, count( $products ) );
		$this->assertEquals( 1, count( $coupons['zyxw'] ) );
		$this->assertEquals( 'unitSupplier', $product->getSupplierCode() );
		$this->assertEquals( 'U:PD', $product->getProductCode() );
		$this->assertNotEquals( '', $product->getProductId() );
		$this->assertEquals( '', $product->getMediaUrl() );
		$this->assertEquals( 'Geschenk Nachlass', $product->getName() );
	}


	public function testDeleteCoupon()
	{
		$this->_object->addCoupon( $this->_orderBase );
		$this->_object->deleteCoupon( $this->_orderBase );

		$coupons = $this->_orderBase->getCoupons();
		$products = $this->_orderBase->getProducts();

		$this->assertEquals( 0, count( $products ) );
		$this->assertArrayNotHasKey( 'zyxw', $coupons );
	}


	public function testAddCouponInvalidConfig()
	{
		$context = TestHelper::getContext();
		$couponItem = MShop_Coupon_Manager_Factory::createManager( TestHelper::getContext() )->createItem();

		$object = new MShop_Coupon_Provider_Present( $context, $couponItem, 'zyxw' );

		$this->setExpectedException( 'MShop_Coupon_Exception' );
		$object->addCoupon( $this->_orderBase );
	}


	public function testIsAvailable()
	{
		$this->assertTrue( $this->_object->isAvailable( $this->_orderBase ) );
	}
}
